@@ -5,6 +5,14 @@ if [ -n $1 ]; then
 fi
 
 quilt series | tac | while read file; do
-	patch -R -p1 < patches/$file;
+	case $file in
+		patches/*)
+			;;
+		*)
+			file="patches/$file"
+			;;
+	esac
+
+	patch -R -p1 < $file;
 done
 quilt push -a
