@@ -27,15 +27,14 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <core/printf.h>
 #include "constants.h"
 #include "convert.h"
 #include "cpu_emul.h"
 #include "cpu_mmu.h"
 #include "cpu_stack.h"
 #include "current.h"
-#include "regs.h"
 #include "panic.h"
+#include "printf.h"
 
 void
 cpu_emul_cpuid (void)
@@ -112,9 +111,10 @@ cpu_emul_sti (void)
 	current->vmctl.read_flags (&rflags);
 	rflags |= RFLAGS_IF_BIT;
 	current->vmctl.write_flags (rflags);
+	current->exint.int_enabled ();
 }
 
-vmmerr_t
+enum vmmerr
 cpu_emul_realmode_int (u8 num)
 {
 	u16 seg, off;
@@ -131,6 +131,18 @@ cpu_emul_realmode_int (u8 num)
 	current->vmctl.read_flags (&rflags);
 	current->vmctl.read_sreg_sel (SREG_CS, &cs);
 	current->vmctl.read_ip (&rip);
+#ifdef DISABLE_TCG_BIOS
+	if (num == 0x1A) {
+		ulong rax;
+
+		current->vmctl.read_general_reg (GENERAL_REG_RAX, &rax);
+		rax &= 0xFFFF;
+		if (rax == 0xBB00) {	/* TCG_StatusCheck function */
+			current->vmctl.write_flags (rflags | RFLAGS_CF_BIT);
+			return VMMERR_SUCCESS;
+		}
+	}
+#endif /* DISABLE_TCG_BIOS */
 	RIE (cpu_stack_get (&st));
 	RIE (cpu_stack_push (&st, &rflags, OPTYPE_16BIT));
 	RIE (cpu_stack_push (&st, &cs, OPTYPE_16BIT));
@@ -144,8 +156,7 @@ cpu_emul_realmode_int (u8 num)
 	return VMMERR_SUCCESS;
 }
 
-/* Copy from bitvisor 1.3 and modify */
-void
+bool
 cpu_emul_xsetbv (void)
 {
 	u32 ia, ic, id;
@@ -157,5 +168,5 @@ cpu_emul_xsetbv (void)
 	ia = la;
 	ic = lc;
 	id = ld;
-	asm_xsetbv (ic, ia, id);
+	return current->vmctl.xsetbv (ic, ia, id);
 }
