@@ -26,13 +26,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+/*
+ * Copyright (c) 2014 Yuichi Watanabe
+ */
 
-#include <core/cpu.h>
-#include <core/initfunc.h>
-#include <core/types.h>
-#include <core/printf.h>
-#include <core/string.h>
 #include "initfunc.h"
+#include "printf.h"
+#include "string.h"
+#include "types.h"
 
 /* #define INIT_DEBUG */
 #ifdef INIT_DEBUG
@@ -84,11 +85,11 @@ call_initfunc (char *id)
 	l = strlen (id);
 	for (p = __initfunc_start; p != __initfunc_end; p++)
 		if (memcmp (p->id, id, l) == 0) {
-			INIT_DBG("CPU%d: Calling %s %p\n",
-				 get_cpu_id(), p->id, p->func);
+			INIT_DBG("Calling %s %p\n",
+				 p->id, p->func);
 			p->func ();
-			INIT_DBG("CPU%d: Returned from %s %p\n",
-				 get_cpu_id(), p->id, p->func);
+			INIT_DBG("Returned from %s %p\n",
+				 p->id, p->func);
 		}
 }
 
