@@ -448,6 +448,7 @@ alloc_realmodemem (uint len)
 	err = rm_insert_resource(rm_find_resource(&mem_address_space, start),
 				 resource);
 	if (err) {
+		rm_dump_resources(&mem_address_space);
 		panic("Failed to insert the allocated realmodemem to phys_mem_map. "
 		      "start 0x%x len 0x%x err 0x%x",
 		      start, len, err);
@@ -842,7 +843,7 @@ mm_init_mem_address_space()
 			 vmm_term_inf() - 1, RESOURCE_TYPE_MEM,
 			 MEM_TYPE_AVAILABLE, "vmm");
 
-	for (nn = 1; nn; n = nn) {
+	for (n = 0, nn = 1; nn; n = nn) {
 		entry = phys_mem_map + index++;
 
 		nn = getsysmemmap (n, &base, &len, &type);
@@ -880,11 +881,11 @@ mm_init_global (void)
 	spinlock_init (&mm_lock2);
 	spinlock_init (&mm_lock_process_virt_to_phys);
 	spinlock_init (&mapmem_lock);
+	getallsysmemmap ();
 	if (uefi_booted) {
 		create_vmm_pd ();
 		asm_wrcr3 (vmm_base_cr3);
 	} else {
-		getallsysmemmap ();
 		find_realmodemem ();
 		vmm_start_phys = find_vmm_phys ();
 		if (vmm_start_phys == 0) {
@@ -2477,30 +2478,6 @@ init_blank_page(void)
 	memset (tmp, 0, PAGESIZE);
 }
 
-void
-get_tmpbuf (u32 *tmpbufaddr, u32 *tmpbufsize)
-{
-	u32 n, type;
-	u64 base, len;
-
-	*tmpbufaddr = callrealmode_endofcodeaddr ();
-	n = 0;
-	do {
-		n = getfakesysmemmap (n, &base, &len, &type);
-		if (type != SYSMEMMAP_TYPE_AVAILABLE)
-			continue;
-		if (base > *tmpbufaddr)
-			continue;
-		if (base + len <= *tmpbufaddr)
-			continue;
-		if (base + len > 0xA0000)
-			len = 0xA0000 - base;
-		*tmpbufsize = base + len - *tmpbufaddr;
-		return;
-	} while (n);
-	panic ("tmpbuf not found");
-}
-
 INITFUNC ("global2", mm_init_global);
 INITFUNC ("ap0", unmap_user_area);
 INITFUNC ("pass0", init_pte_addr_mask);
