@@ -26,42 +26,44 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-/*
- * Copyright (c) 2010-2014 Yuichi Watanabe
- */
 
-#include <core/assert.h>
-#include <core/extint.h>
-#include <core/initfunc.h>
-#include <core/linkage.h>
-#include <core/printf.h>
-#include <core/string.h>
-#include "apic_pass.h"
-#include "apic.h"
+#include <core/exint.h>
 #include "asm.h"
-#include "bitmap.h"
+#include "apic.h"
 #include "constants.h"
-#include "cpu.h"
+#include "convert.h"
 #include "cpu_emul.h"
 #include "cpu_mmu.h"
 #include "current.h"
-#include "extint.h"
-#include "guest_boot.h"
+#include "exint_pass.h"
+#include "gmm_pass.h"
+#include "initfunc.h"
 #include "int.h"
+#include "linkage.h"
 #include "panic.h"
 #include "pcpu.h"
+#include "printf.h"
 #include "reboot.h"
+#include "string.h"
 #include "thread.h"
-#include "vcpu.h"
 #include "vmmcall.h"
 #include "vmmcall_status.h"
 #include "vt.h"
-#include "vt_internal.h"
+#include "vt_addip.h"
+#include "vt_exitreason.h"
+#include "vt_init.h"
+#include "vt_io.h"
+#include "vt_main.h"
+#include "vt_paging.h"
+#include "vt_regs.h"
+#include "vt_vmcs.h"
 
-enum vt_status {
-	VT_VMENTRY_SUCCESS,
-	VT_VMENTRY_FAILED,
-	VT_VMEXIT,
+#define EPT_VIOLATION_EXIT_QUAL_WRITE_BIT 0x2
+
+enum vt__status {
+	VT__VMENTRY_SUCCESS,
+	VT__VMENTRY_FAILED,
+	VT__VMEXIT,
 };
 
 static u32 stat_intcnt = 0;
@@ -102,7 +104,7 @@ do_mov_cr (void)
 		vt_write_control_reg (CONTROL_REG_CR0, val);
 		break;
 	default:
-		panic ("Not implemented.");
+		panic ("Fatal error: Not implemented.");
 	}
 	add_ip ();
 }
@@ -115,32 +117,25 @@ do_cpuid (void)
 }
 
 static void
-do_msr_fault (void)
+make_gp_fault (u32 errcode)
 {
 	struct vt_intr_data *vid = &current->u.vt.intr;
 
-	if (vid->intr_info.s.valid == INTR_INFO_VALID_VALID) {
-		panic ("Can't inject general protection fault (MSR) because the other event is already pending. "
-		       "vector 0x%x type 0x%x, err_valid 0x%x",
-					       vid->intr_info.s.vector,
-					       vid->intr_info.s.type,
-					       vid->intr_info.s.err);
-
-	}
-	vid->intr_info.v = 0;
-	vid->intr_info.s.vector = EXCEPTION_GP;
-	vid->intr_info.s.type = INTR_INFO_TYPE_HARD_EXCEPTION;
-	vid->intr_info.s.err = INTR_INFO_ERR_VALID;
-	vid->intr_info.s.valid = INTR_INFO_VALID_VALID;
-	vid->exception_errcode = 0;
-	vid->instruction_len = 0;
+	vid->vmcs_intr_info.s.vector = EXCEPTION_GP;
+	vid->vmcs_intr_info.s.type = INTR_INFO_TYPE_HARD_EXCEPTION;
+	vid->vmcs_intr_info.s.err = INTR_INFO_ERR_VALID;
+	vid->vmcs_intr_info.s.nmi = 0;
+	vid->vmcs_intr_info.s.reserved = 0;
+	vid->vmcs_intr_info.s.valid = INTR_INFO_VALID_VALID;
+	vid->vmcs_exception_errcode = errcode;
+	vid->vmcs_instruction_len = 0;
 }
 
 static void
 do_rdmsr (void)
 {
 	if (cpu_emul_rdmsr ())
-		do_msr_fault ();
+		make_gp_fault (0);
 	else
 		add_ip ();
 }
@@ -149,11 +144,24 @@ static void
 do_wrmsr (void)
 {
 	if (cpu_emul_wrmsr ())
-		do_msr_fault ();
+		make_gp_fault (0);
 	else
 		add_ip ();
 }
 
+void
+vt_update_exception_bmp (void)
+{
+	u32 newbmp = 0xFFFFFFFF;
+
+	if (!current->u.vt.vr.re && !current->u.vt.vr.sw.enable) {
+		newbmp = 1 << EXCEPTION_NMI;
+		if (current->u.vt.handle_pagefault)
+			newbmp |= 1 << EXCEPTION_PF;
+	}
+	asm_vmwrite (VMCS_EXCEPTION_BMP, newbmp);
+}
+
 static void
 do_exception (void)
 {
@@ -162,116 +170,99 @@ do_exception (void)
 		ulong v;
 	} vii;
 	ulong len;
-	vmmerr_t err;
+	enum vmmerr err;
 	ulong errc;
 
 	asm_vmread (VMCS_VMEXIT_INTR_INFO, &vii.v);
-	if (vii.s.valid != INTR_INFO_VALID_VALID) {
-		return;
-	}
-	switch (vii.s.type) {
-	case INTR_INFO_TYPE_HARD_EXCEPTION:
-		STATUS_UPDATE (asm_lock_incl (&stat_hwexcnt));
-		if (vii.s.vector == EXCEPTION_DB &&
-		    current->u.vt.vr.sw.enable) {
-			break;
-		}
-		if (vii.s.vector == EXCEPTION_PF) {
-			ulong cr2;
-			asm_vmread (VMCS_VMEXIT_INTR_ERRCODE, &errc);
-			asm_vmread (VMCS_EXIT_QUALIFICATION, &cr2);
-
-			if (current->u.vt.ept_enabled && current->u.vt.vr.pg) {
-				/*
-				 * page fault causes VM-exit
-				 * even though EPT is enabled and page
-				 * fault is masked.
-				 */
-				if (current->u.vt.intr.intr_info.s.valid
-				    == INTR_INFO_VALID_VALID) {
-					panic ("Can't inject a page fault because the other event is already pending. "
-					       "vector 0x%x type 0x%x, err_valid 0x%x",
-					       current->u.vt.intr.intr_info.s.vector,
-					       current->u.vt.intr.intr_info.s.type,
-					       current->u.vt.intr.intr_info.s.err);
-				}
-				current->u.vt.intr.intr_info.v = vii.v;
-				current->u.vt.vr.cr2 = cr2;
-				current->u.vt.intr.exception_errcode = errc;
+	if (vii.s.valid == INTR_INFO_VALID_VALID) {
+		switch (vii.s.type) {
+		case INTR_INFO_TYPE_HARD_EXCEPTION:
+			STATUS_UPDATE (asm_lock_incl (&stat_hwexcnt));
+			if (vii.s.vector == EXCEPTION_DB &&
+			    current->u.vt.vr.sw.enable)
 				break;
-			}
+			if (vii.s.vector == EXCEPTION_PF) {
+				ulong err, cr2;
 
-			cpu_mmu_spt_pagefault (errc, cr2);
-			STATUS_UPDATE (asm_lock_incl (&stat_pfcnt));
-		} else if (!current->u.vt.vr.pe) {
-			/* Real mode */
-			switch (vii.s.vector) {
-			case EXCEPTION_GP:
-				err = cpu_interpreter ();
-				if (err == VMMERR_SUCCESS)
+				asm_vmread (VMCS_VMEXIT_INTR_ERRCODE, &err);
+				asm_vmread (VMCS_EXIT_QUALIFICATION, &cr2);
+				vt_paging_pagefault (err, cr2);
+				STATUS_UPDATE (asm_lock_incl (&stat_pfcnt));
+			} else if (current->u.vt.vr.re) {
+				switch (vii.s.vector) {
+				case EXCEPTION_GP:
+					err = cpu_interpreter ();
+					if (err == VMMERR_SUCCESS)
+						break;
+					panic ("Fatal error:"
+					       " General protection fault"
+					       " in real mode."
+					       " (err: %d)", err);
+				case EXCEPTION_DB:
+					if (cpu_emul_realmode_int (1)) {
+						panic ("Fatal error:"
+						       " realmode_int"
+						       " error"
+						       " (int 1)");
+					}
 					break;
-				panic ("General protection fault in real mode and can't emulate the instruction (err: %d)",
-				       err);
-			case EXCEPTION_DB:
-				if (cpu_emul_realmode_int (1)) {
-					panic ("cpu_emul_realmode_int (1) error");
+				default:
+					panic ("Fatal error:"
+					       " Unimplemented vector 0x%X",
+					       vii.s.vector);
 				}
-				break;
-			default:
-				panic ("Unimplemented exception in real mode: "
-				       "0x%02X %s\n", vii.s.vector,
-				       exception_name (vii.s.vector));
-			}
-		} else {
-			if (current->u.vt.intr.intr_info.s.valid == INTR_INFO_VALID_VALID) {
-				panic ("Can't inject a exception because the other event is already pending. "
-				       "old: vector 0x%x type 0x%x, err_valid 0x%x "
-				       "new: vector 0x%x type 0x%x, err_valid 0x%x",
-				       current->u.vt.intr.intr_info.s.vector,
-				       current->u.vt.intr.intr_info.s.type,
-				       current->u.vt.intr.intr_info.s.err,
-				       vii.s.vector, vii.s.type, vii.s.err);
-			}
-#ifdef VT_EXCEPTION_DEBUG
-			if ((current->u.vt.intr.intr_info.v | 0x8000000)
-			    == vii.v) {
-				printf("DEBUG: Injecting the same exception twice.\n");
-			}
-#endif
-			current->u.vt.intr.intr_info.v = vii.v;
-			if (vii.s.err == INTR_INFO_ERR_VALID) {
-				asm_vmread (VMCS_VMEXIT_INTR_ERRCODE,
-					    &errc);
-				current->u.vt.intr.exception_errcode = errc;
-#ifdef VT_EXCEPTION_DEBUG
-				if (vii.s.vector == EXCEPTION_GP) {
-					printf ("DEBUG: General protection fault on CPU%d. error code 0x%lx\n", get_cpu_id(), errc);
+			} else {
+				if (current->u.vt.intr.vmcs_intr_info.v
+				    == vii.v)
+					panic ("Double fault"
+					       " in do_exception.");
+				current->u.vt.intr.vmcs_intr_info.v = vii.v;
+				if (vii.s.err == INTR_INFO_ERR_VALID) {
+					asm_vmread (VMCS_VMEXIT_INTR_ERRCODE,
+						    &errc);
+					current->u.vt.intr.
+						vmcs_exception_errcode = errc;
+					if (vii.s.vector == EXCEPTION_GP &&
+					    errc == 0x6B)
+						panic ("Fatal error:"
+						       " General protection"
+						       " fault"
+						       " (maybe double"
+						       " fault).");
 				}
-#endif
+				current->u.vt.intr.vmcs_instruction_len = 0;
+#if 0				/* Exception monitoring test */
+				if (vii.s.vector == EXCEPTION_DE) {
+					u32 cs, eip;
+
+					asm_vmread (VMCS_GUEST_CS_SEL, &cs);
+					asm_vmread (VMCS_GUEST_RIP, &eip);
+					printf ("Exception monitor test:"
+						" Devide Error Exception"
+						" at 0x%x:0x%x\n",
+						cs, eip);
+				}
+#endif				/* Exception monitoring test */
 			}
-			current->u.vt.intr.instruction_len = 0;
+			break;
+		case INTR_INFO_TYPE_SOFT_EXCEPTION:
+			STATUS_UPDATE (asm_lock_incl (&stat_swexcnt));
+			current->u.vt.intr.vmcs_intr_info.v = vii.v;
+			asm_vmread (VMCS_VMEXIT_INSTRUCTION_LEN, &len);
+			current->u.vt.intr.vmcs_instruction_len = len;
+			break;
+		case INTR_INFO_TYPE_NMI:
+			vii.s.nmi = 0; /* FIXME */
+			current->u.vt.intr.vmcs_intr_info.v = vii.v;
+			asm_vmread (VMCS_VMEXIT_INSTRUCTION_LEN, &len);
+			current->u.vt.intr.vmcs_instruction_len = len;
+			break;
+		case INTR_INFO_TYPE_EXTERNAL:
+		default:
+			panic ("Fatal error:"
+			       " intr_info_type %d not implemented",
+			       vii.s.type);
 		}
-		break;
-	case INTR_INFO_TYPE_SOFT_EXCEPTION:
-		if (current->u.vt.intr.intr_info.s.valid == INTR_INFO_VALID_VALID) {
-			panic ("Can't inject a soft exception because the other event is already pending. "
-			       "vector 0x%x type 0x%x, err_valid 0x%x",
-			       current->u.vt.intr.intr_info.s.vector,
-			       current->u.vt.intr.intr_info.s.type,
-			       current->u.vt.intr.intr_info.s.err);
-		}
-		STATUS_UPDATE (asm_lock_incl (&stat_swexcnt));
-		current->u.vt.intr.intr_info.v = vii.v;
-		asm_vmread (VMCS_VMEXIT_INSTRUCTION_LEN, &len);
-		current->u.vt.intr.instruction_len = len;
-		break;
-	case INTR_INFO_TYPE_NMI:
-		panic("NMI recieved while executing guest software.");
-		break;
-	case INTR_INFO_TYPE_EXTERNAL:
-	default:
-		panic ("intr_info_type %d not implemented",
-		       vii.s.type);
 	}
 }
 
@@ -281,16 +272,10 @@ do_invlpg (void)
 	ulong linear;
 
 	asm_vmread (VMCS_EXIT_QUALIFICATION, &linear);
-	cpu_mmu_spt_invalidate (linear);
+	vt_paging_invalidate (linear);
 	add_ip ();
 }
 
-void
-vt_invlpg (ulong addr)
-{
-	cpu_mmu_spt_invalidate (addr);
-}
-
 /* VMCALL: guest calls VMM */
 static void
 do_vmcall (void)
@@ -299,97 +284,185 @@ do_vmcall (void)
 	vmmcall ();
 }
 
-
 static void
-vt_event_delivery_setup (void)
+vt__nmi (void)
 {
 	struct vt_intr_data *vid = &current->u.vt.intr;
 
-	if (vid->intr_info.s.valid == INTR_INFO_VALID_VALID) {
-		asm_vmwrite (VMCS_VMENTRY_INTR_INFO_FIELD,
-			     vid->intr_info.v);
-		if (vid->intr_info.s.err == INTR_INFO_ERR_VALID)
-			asm_vmwrite (VMCS_VMENTRY_EXCEPTION_ERRCODE,
-				     vid->exception_errcode);
-		asm_vmwrite (VMCS_VMENTRY_INSTRUCTION_LEN,
-			     vid->instruction_len);
-	}
-}
-
-static enum vt_status
-call_vt_vmlaunch (void)
-{
-	if (asm_vmlaunch_regs (&current->u.vt.vr))
-		return VT_VMENTRY_FAILED;
-	return VT_VMEXIT;
-}
-
-static enum vt_status
-call_vt_vmresume (void)
-{
-	if (asm_vmresume_regs (&current->u.vt.vr))
-		return VT_VMENTRY_FAILED;
-	return VT_VMEXIT;
+	if (vid->vmcs_intr_info.s.valid == INTR_INFO_VALID_VALID)
+		return;
+	if (!current->nmi.get_nmi_count ())
+		return;
+	if (current->u.vt.vr.re)
+		panic ("NMI in real mode");
+	vid->vmcs_intr_info.v = 0;
+	vid->vmcs_intr_info.s.vector = EXCEPTION_NMI;
+	vid->vmcs_intr_info.s.type = INTR_INFO_TYPE_NMI;
+	vid->vmcs_intr_info.s.err = INTR_INFO_ERR_INVALID;
+	vid->vmcs_intr_info.s.valid = INTR_INFO_VALID_VALID;
 }
 
 static void
-vt_vm_run (void)
+vt__event_delivery_setup (void)
 {
-	enum vt_status status;
+	struct vt_intr_data *vid = &current->u.vt.intr;
 
-	if (!current->u.vt.launched) {
-		current->u.vt.launched = true;
-		status = call_vt_vmlaunch ();
-	} else {		
-		status = call_vt_vmresume ();
+	if (vid->vmcs_intr_info.s.valid == INTR_INFO_VALID_VALID) {
+		asm_vmwrite (VMCS_VMENTRY_INTR_INFO_FIELD,
+			     vid->vmcs_intr_info.v);
+		if (vid->vmcs_intr_info.s.err == INTR_INFO_ERR_VALID)
+			asm_vmwrite (VMCS_VMENTRY_EXCEPTION_ERRCODE,
+				     vid->vmcs_exception_errcode);
+		asm_vmwrite (VMCS_VMENTRY_INSTRUCTION_LEN,
+			     vid->vmcs_instruction_len);
 	}
-	if (status != VT_VMEXIT) {
-		if (status == VT_VMENTRY_FAILED)
-			panic ("VM entry failed.");
+}
+
+static enum vt__status
+call_vt__vmlaunch (void)
+{
+	if (asm_vmlaunch_regs (&current->u.vt.vr))
+		return VT__VMENTRY_FAILED;
+	return VT__VMEXIT;
+}
+
+static enum vt__status
+call_vt__vmresume (void)
+{
+	if (asm_vmresume_regs (&current->u.vt.vr))
+		return VT__VMENTRY_FAILED;
+	return VT__VMEXIT;
+}
+
+static void
+vt__vm_run_first (void)
+{
+	enum vt__status status;
+
+	status = call_vt__vmlaunch ();
+	if (status != VT__VMEXIT) {
+		if (status == VT__VMENTRY_FAILED)
+			panic ("Fatal error: VM entry failed.");
 		else
-			panic ("Strange status.");
+			panic ("Fatal error: Strange status.");
+	}
+}
+
+static void
+vt__vm_run (void)
+{
+	enum vt__status status;
+
+	if (current->u.vt.first) {
+		vt__vm_run_first ();
+		current->u.vt.first = false;
+		return;
+	}
+	if (current->u.vt.exint_update)
+		vt_update_exint ();
+	if (current->u.vt.saved_vmcs)
+		spinlock_unlock (&currentcpu->suspend_lock);
+	status = call_vt__vmresume ();
+	if (current->u.vt.saved_vmcs)
+		spinlock_lock (&currentcpu->suspend_lock);
+	if (status != VT__VMEXIT) {
+		if (status == VT__VMENTRY_FAILED)
+			panic ("Fatal error: VM entry failed.");
+		else
+			panic ("Fatal error: Strange status.");
 	}
 }
 
 /* FIXME: bad handling of TF bit */
 static void
-vt_vm_run_with_tf (void)
+vt__vm_run_with_tf (void)
 {
 	ulong rflags;
 
 	vt_read_flags (&rflags);
 	rflags |= RFLAGS_TF_BIT;
 	vt_write_flags (rflags);
-	vt_vm_run ();
+	vt__vm_run ();
 	vt_read_flags (&rflags);
 	rflags &= ~RFLAGS_TF_BIT;
 	vt_write_flags (rflags);
 }
 
 static void
-vt_event_delivery_check (void)
+vt__event_delivery_check (void)
 {
+	ulong err;
 	union {
 		struct intr_info s;
 		ulong v;
 	} ivif;
 	struct vt_intr_data *vid = &current->u.vt.intr;
 
-	if (vid->intr_info.s.valid == INTR_INFO_VALID_VALID) {
-		/*
-		 * Check if VM Exits occured during event delivery.
-		 */
-		asm_vmread (VMCS_IDT_VECTORING_INFO_FIELD, &ivif.v);
-		if (ivif.s.valid == INTR_INFO_VALID_INVALID) {
-			vid->intr_info.s.valid = INTR_INFO_VALID_INVALID;
-		} else if (ivif.v != vid->intr_info.v) {
-			panic (
-			       "VMCS_IDT_VECTORING_INFO 0x%lX "
-			       "is different from "
-			       "VMCS_VMENTRY_INTR_INFO before VM-enter 0x%X\n",
-			       ivif.v, vid->intr_info.v);
+	/* The IDT-vectoring information field has event information
+	   that is not delivered yet. The event will be the next
+	   event if no other events will have been injected.
+	   This field may or may not be the same as the VM-entry
+	   interruption-information field. Atom Z520/Z530 seems to
+	   behave differently from other processors about this field. */
+	asm_vmread (VMCS_IDT_VECTORING_INFO_FIELD, &ivif.v);
+	if (ivif.s.valid == INTR_INFO_VALID_VALID) {
+		if (ivif.s.type == INTR_INFO_TYPE_SOFT_INTR ||
+		    ivif.s.type == INTR_INFO_TYPE_SOFT_EXCEPTION) {
+			/* Ignore software interrupt to
+			   make this function simple.
+			   The INT instruction will be executed again. */
+			ivif.s.valid = INTR_INFO_VALID_INVALID;
+		} else if (ivif.s.err == INTR_INFO_ERR_VALID) {
+			asm_vmread (VMCS_IDT_VECTORING_ERRCODE, &err);
+			vid->vmcs_exception_errcode = err;
 		}
 	}
+	vid->vmcs_intr_info.v = ivif.v;
+}
+
+static void
+do_init_signal (void)
+{
+	if (currentcpu->cpunum == 0)
+		handle_init_to_bsp ();
+
+	asm_vmwrite (VMCS_GUEST_ACTIVITY_STATE,
+		     VMCS_GUEST_ACTIVITY_STATE_WAIT_FOR_SIPI);
+	current->halt = false;
+	current->u.vt.vr.sw.enable = 0;
+	vt_update_exception_bmp ();
+
+	if (vm_resetting()) {
+		vm_wait_other_cpus();
+	}
+}
+
+static void
+do_startup_ipi (void)
+{
+	ulong vector;
+
+	asm_vmread (VMCS_EXIT_QUALIFICATION, &vector);
+	vector &= 0xFF;
+	vt_reset ();
+	vt_write_realmode_seg (SREG_CS, vector << 8);
+	vt_write_general_reg (GENERAL_REG_RAX, 0);
+	vt_write_general_reg (GENERAL_REG_RCX, 0);
+	vt_write_general_reg (GENERAL_REG_RDX, 0);
+	vt_write_general_reg (GENERAL_REG_RBX, 0);
+	vt_write_general_reg (GENERAL_REG_RSP, 0);
+	vt_write_general_reg (GENERAL_REG_RBP, 0);
+	vt_write_general_reg (GENERAL_REG_RSI, 0);
+	vt_write_general_reg (GENERAL_REG_RDI, 0);
+	vt_write_ip (0);
+	vt_write_flags (RFLAGS_ALWAYS1_BIT);
+	vt_write_idtr (0, 0x3FF);
+	asm_vmwrite (VMCS_GUEST_ACTIVITY_STATE,
+		     VMCS_GUEST_ACTIVITY_STATE_ACTIVE);
+	vt_update_exception_bmp ();
+
+	apic_maskall();
+	exint_reset_apic();
 }
 
 static void
@@ -408,7 +481,7 @@ task_switch_load_segdesc (u16 sel, ulong gdtr_base, ulong gdtr_limit,
 		struct segdesc s;
 		u64 v;
 	} desc;
-	vmmerr_t r;
+	enum vmmerr r;
 
 	/* FIXME: set busy bit */
 	if (sel == 0)
@@ -443,7 +516,7 @@ task_switch_load_segdesc (u16 sel, ulong gdtr_base, ulong gdtr_limit,
 static void
 do_task_switch (void)
 {
-	vmmerr_t r;
+	enum vmmerr r;
 	union {
 		struct exit_qual_ts s;
 		ulong v;
@@ -569,14 +642,39 @@ do_task_switch (void)
 	if (r != VMMERR_SUCCESS)
 		goto err;
 	/* load segment descriptors */
-	if (rflags & RFLAGS_VM_BIT)
-		panic ("switching to virtual 8086 mode");
 	task_switch_load_segdesc (eqt.s.sel, gdtr_base, gdtr_limit,
 				  VMCS_GUEST_TR_BASE, VMCS_GUEST_TR_LIMIT,
 				  VMCS_GUEST_TR_ACCESS_RIGHTS);
 	task_switch_load_segdesc (tss32_2.ldt, gdtr_base, gdtr_limit,
 				  VMCS_GUEST_LDTR_BASE, VMCS_GUEST_LDTR_LIMIT,
 				  VMCS_GUEST_LDTR_ACCESS_RIGHTS);
+	if (rflags & RFLAGS_VM_BIT) {
+		asm_vmwrite (VMCS_GUEST_ES_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_CS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_SS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_ES_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_CS_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_SS_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_DS_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_FS_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_GS_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_ES_SEL, tss32_2.es);
+		asm_vmwrite (VMCS_GUEST_ES_BASE, tss32_2.es << 4);
+		asm_vmwrite (VMCS_GUEST_CS_SEL, tss32_2.cs);
+		asm_vmwrite (VMCS_GUEST_CS_BASE, tss32_2.cs << 4);
+		asm_vmwrite (VMCS_GUEST_SS_SEL, tss32_2.ss);
+		asm_vmwrite (VMCS_GUEST_SS_BASE, tss32_2.ss << 4);
+		asm_vmwrite (VMCS_GUEST_DS_SEL, tss32_2.ds);
+		asm_vmwrite (VMCS_GUEST_DS_BASE, tss32_2.ds << 4);
+		asm_vmwrite (VMCS_GUEST_FS_SEL, tss32_2.fs);
+		asm_vmwrite (VMCS_GUEST_FS_BASE, tss32_2.fs << 4);
+		asm_vmwrite (VMCS_GUEST_GS_SEL, tss32_2.gs);
+		asm_vmwrite (VMCS_GUEST_GS_BASE, tss32_2.gs << 4);
+		goto virtual8086mode;
+	}
 	task_switch_load_segdesc (tss32_2.es, gdtr_base, gdtr_limit,
 				  VMCS_GUEST_ES_BASE, VMCS_GUEST_ES_LIMIT,
 				  VMCS_GUEST_ES_ACCESS_RIGHTS);
@@ -595,31 +693,91 @@ do_task_switch (void)
 	task_switch_load_segdesc (tss32_2.gs, gdtr_base, gdtr_limit,
 				  VMCS_GUEST_GS_BASE, VMCS_GUEST_GS_LIMIT,
 				  VMCS_GUEST_GS_ACCESS_RIGHTS);
+virtual8086mode:
 	vt_read_control_reg (CONTROL_REG_CR0, &tmp);
 	tmp |= CR0_TS_BIT;
 	vt_write_control_reg (CONTROL_REG_CR0, tmp);
+	/* When source of the task switch is an interrupt, intr info
+	 * which may contain information of the interrupt needs to be
+	 * cleared. */
+	current->u.vt.intr.vmcs_intr_info.v = 0;
 	return;
 err:
 	panic ("do_task_switch: error %d", r);
 }
 
-/* Copy from bitvisor 1.3 and modify */
 static void
 do_xsetbv (void)
 {
-	cpu_emul_xsetbv();
-	add_ip ();
+	u16 cs;
+
+	/* According to the manual, XSETBV causes a VM exit regardless
+	 * of the value of CPL.  Maybe it is different from the real
+	 * behavior, but check CPL here to be sure. */
+	vt_read_sreg_sel (SREG_CS, &cs);
+	if ((cs & 3) || cpu_emul_xsetbv ())
+		make_gp_fault (0);
+	else
+		add_ip ();
 }
 
 static void
-vt_exit_reason (void)
+do_ept_violation (void)
+{
+	ulong eqe;
+	u64 gp;
+
+	asm_vmread (VMCS_EXIT_QUALIFICATION, &eqe);
+	asm_vmread64 (VMCS_GUEST_PHYSICAL_ADDRESS, &gp);
+	vt_paging_npf (!!(eqe & EPT_VIOLATION_EXIT_QUAL_WRITE_BIT), gp);
+}
+
+static void
+do_re_external_int (void)
+{
+	ulong rflags;
+	int num;
+
+	vt_read_flags (&rflags);
+	if (rflags & RFLAGS_IF_BIT) {
+		num = do_externalint_enable ();
+		if (num >= 0)
+			vt_generate_external_int (num);
+		current->u.vt.exint_re_pending = false;
+		current->u.vt.exint_update = true;
+	} else {
+		current->u.vt.exint_re_pending = true;
+		current->u.vt.exint_update = true;
+	}
+}
+
+static void
+do_external_int (void)
+{
+	if (current->u.vt.vr.re && current->u.vt.exint_pass)
+		do_re_external_int ();
+	else
+		do_exint_pass ();
+}
+
+static void
+do_interrupt_window (void)
+{
+	if (current->u.vt.exint_re_pending && current->u.vt.vr.re &&
+	    current->u.vt.exint_pass)
+		do_re_external_int ();
+	if (current->u.vt.exint_pending)
+		current->exint.hlt ();
+}
+
+static void
+vt__exit_reason (void)
 {
 	ulong exit_reason;
 
 	asm_vmread (VMCS_EXIT_REASON, &exit_reason);
-	if (exit_reason & EXIT_REASON_VMENTRY_FAILURE_BIT) {
-		panic ("VM Entry failure.");
-	}
+	if (exit_reason & EXIT_REASON_VMENTRY_FAILURE_BIT)
+		panic ("Fatal error: VM Entry failure.");
 	switch (exit_reason & EXIT_REASON_MASK) {
 	case EXIT_REASON_MOV_CR:
 		do_mov_cr ();
@@ -642,18 +800,10 @@ vt_exit_reason (void)
 		break;
 	case EXIT_REASON_EXTERNAL_INT:
 		STATUS_UPDATE (asm_lock_incl (&stat_intcnt));
-		/*
-		 * extint_check_interrupt will be inject the pending
-		 * interrupt or keep it.
-		 */
-		extint_request_check();
+		do_external_int ();
 		break;
 	case EXIT_REASON_INTERRUPT_WINDOW:
-		/*
-		 * extint_check_interrupt will be inject the pending
-		 * interrupt.
-		 */
-		extint_request_check();
+		do_interrupt_window ();
 		break;
 	case EXIT_REASON_INVLPG:
 		do_invlpg ();
@@ -661,6 +811,12 @@ vt_exit_reason (void)
 	case EXIT_REASON_VMCALL: /* for debugging */
 		do_vmcall ();
 		break;
+	case EXIT_REASON_INIT_SIGNAL:
+		do_init_signal ();
+		break;
+	case EXIT_REASON_STARTUP_IPI:
+		do_startup_ipi ();
+		break;
 	case EXIT_REASON_HLT:
 		STATUS_UPDATE (asm_lock_incl (&stat_hltcnt));
 		do_hlt ();
@@ -668,48 +824,21 @@ vt_exit_reason (void)
 	case EXIT_REASON_TASK_SWITCH:
 		do_task_switch ();
 		break;
-	case EXIT_REASON_INIT_SIGNAL:
-		if (cpu_is_bsp()) {
-			/*
-			 * Some controller (For example, ACPI
-			 * controller) might asserted INIT signal.
-			 * To reboot machine, we seem to need to
-			 * execute vmxoff on CPU0.
-			 */
-			asm_vmxoff();
-			apic_send_init(get_apic_id());
-			freeze();
-		} else {
-			printf("Reset cpu 0x%x\n", get_cpu_id());
-			vcpu_reset();
-			apic_maskall();
-			if (vm_resetting()) {
-				vm_wait_other_cpus();
-				vm_wait_other_cpus();
-			}
-			extint_reset_apic();
-			if (vm_resetting()) {
-				vm_wait_other_cpus();
-			}
-		}
-		break;
 	case EXIT_REASON_XSETBV:
 		do_xsetbv ();
 		break;
 	case EXIT_REASON_EPT_VIOLATION:
-		vt_ept_violation();
+		do_ept_violation ();
 		break;
-	case EXIT_REASON_EPT_MISCONFIG:
-		panic("EPT misconfig");
-		break;
-
 	default:
-		panic ("Unhandled VM exit (0x%lx)", exit_reason);
+		printf ("Fatal error: handler not implemented.\n");
+		printexitreason (exit_reason);
+		panic ("Fatal error: handler not implemented.");
 	}
 }
 
 static void
-vt_halt (void)
+vt__halt (void)
 {
 	struct {
 		ulong sel;
@@ -718,16 +847,9 @@ vt_halt (void)
 	} es, cs, ss, ds, fs, gs;
 	struct vt_intr_data *vid = &current->u.vt.intr;
 	ulong rflags;
-	ulong int_state;
 
-	if (vid->intr_info.s.valid == INTR_INFO_VALID_VALID) {
-		/*
-		 * RIP have been incremented by interpreter,
-		 * so that we just return.
-		 */
+	if (vid->vmcs_intr_info.s.valid == INTR_INFO_VALID_VALID)
 		return;
-	}
-
 	asm_vmread (VMCS_GUEST_ES_SEL, &es.sel);
 	asm_vmread (VMCS_GUEST_CS_SEL, &cs.sel);
 	asm_vmread (VMCS_GUEST_SS_SEL, &ss.sel);
@@ -768,22 +890,7 @@ vt_halt (void)
 	asm_vmwrite (VMCS_GUEST_RFLAGS, RFLAGS_ALWAYS1_BIT | RFLAGS_IOPL_0 |
 		(rflags & RFLAGS_IF_BIT));
 	asm_vmwrite (VMCS_GUEST_ACTIVITY_STATE, VMCS_GUEST_ACTIVITY_STATE_HLT);
-
-	/*
-	 * Avoid blocking.
-	 */
-	asm_vmread (VMCS_GUEST_INTERRUPTIBILITY_STATE, &int_state);
-	int_state &= (VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_STI_BIT |
-		      VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_MOV_SS_BIT);
-	asm_vmwrite (VMCS_GUEST_INTERRUPTIBILITY_STATE, int_state);
-
-	if (rflags & RFLAGS_IF_BIT) {
-		vt_extint_pending(false);
-	} else {
-		vt_extint_pending(true);
-	}
-
-	vt_vm_run ();
+	vt__vm_run ();
 	if (false) {		/* DEBUG */
 		ulong exit_reason;
 
@@ -812,27 +919,26 @@ vt_halt (void)
 	asm_vmwrite (VMCS_GUEST_FS_LIMIT, fs.limit);
 	asm_vmwrite (VMCS_GUEST_GS_LIMIT, gs.limit);
 	asm_vmwrite (VMCS_GUEST_RFLAGS, rflags);
-	vt_event_delivery_check ();
-	vt_exit_reason ();
+	vt__event_delivery_check ();
+	vt__exit_reason ();
 }
 
 static void
 vt_mainloop (void)
 {
-	vmmerr_t err;
+	enum vmmerr err;
 	ulong cr0, acr;
 	u64 efer;
 
 	for (;;) {
 		schedule ();
-		apic_handle_wait_for_sipi_state();
-
+		vt_vmptrld (current->u.vt.vi.vmcs_region_phys);
+		panic_test ();
 		if (current->halt) {
-			vt_halt();
+			vt__halt ();
 			current->halt = false;
 			continue;
 		}
-
 		/* when the state is switching between real mode and
 		   protected mode, we try emulation first */
 		/* SWITCHING:
@@ -855,6 +961,7 @@ vt_mainloop (void)
 				/* consider emulation is not needed after
 				   32 instructions are executed */
 				current->u.vt.vr.sw.enable = 0;
+				vt_update_exception_bmp ();
 				continue;
 			}
 			vt_read_control_reg (CONTROL_REG_CR0, &cr0);
@@ -865,6 +972,7 @@ vt_mainloop (void)
 					if (acr & ACCESS_RIGHTS_L_BIT) {
 						/* long mode */
 						current->u.vt.vr.sw.enable = 0;
+						vt_update_exception_bmp ();
 						continue;
 					}
 				}
@@ -882,19 +990,22 @@ vt_mainloop (void)
 			   (sw.enable may be changed after cpu_interpreter())
 			*/
 		}
-
-		extint_check_interrupt ();
-		vt_event_delivery_setup ();
+		/* when the state is switching, do single step */
 		if (current->u.vt.vr.sw.enable) {
-			/* when the state is switching, do single step */
-			vt_vm_run_with_tf ();
-		} else {
-			/* not switching */
-			vt_vm_run ();
+			vt__nmi ();
+			vt__event_delivery_setup ();
+			vt__vm_run_with_tf ();
+			vt_paging_tlbflush ();
+			vt__event_delivery_check ();
+			vt__exit_reason ();
+		} else {	/* not switching */
+			vt__nmi ();
+			vt__event_delivery_setup ();
+			vt__vm_run ();
+			vt_paging_tlbflush ();
+			vt__event_delivery_check ();
+			vt__exit_reason ();
 		}
-		cpu_mmu_spt_tlbflush ();
-		vt_event_delivery_check ();
-		vt_exit_reason ();
 	}
 }
 
@@ -924,9 +1035,17 @@ vt_register_status_callback (void)
 }
 
 void
-vt_run_vm (void)
+vt_init_signal (void)
 {
+	do_init_signal ();
+}
+
+void
+vt_start_vm (void)
+{
+	current->exint.int_enabled ();
+	vt_paging_start ();
 	vt_mainloop ();
 }
 
-INITFUNC ("global4", vt_register_status_callback);
+INITFUNC ("paral01", vt_register_status_callback);
